#include "stdafx.h"
#include "SigVisionTransferClass.h"

#include <visiontransfer/deviceenumeration.h>
#include <visiontransfer/imagetransfer.h>
#include <visiontransfer/imageset.h>
#include <visiontransfer/deviceparameters.h>
#include <visiontransfer/parameterset.h>
#include <visiontransfer/parametertransfer.h>
#include "visiontransfer/parameterinfo.h"
#include <visiontransfer/datachannelservice.h>
#include <visiontransfer/reconstruct3d.h>

// Constructor
SigVisionClass::SigVisionClass() {
    // Nach Stereo Devices suchen
    visiontransfer::DeviceEnumeration deviceEnum;

    visiontransfer::DeviceEnumeration::DeviceList devices = deviceEnum.discoverDevices();

    if (devices.size() == 0) {
        throw SigException("no devices available");
    }

    SelectedDevice = devices[0];
    IsTransferring = false;

    InitializeCriticalSection(&ImuDatenLock);
    IsUpdatingImu = false;

    setImuDaten(0, 0, 0, 0, 0, 0);
}

// Destructor
SigVisionClass::~SigVisionClass() {
    DeleteCriticalSection(&ImuDatenLock);
}

// IMU Daten auslesen
void SigVisionClass::GetIMU(
    double& _Roll, double& _Pitch, double& _Yaw,
    double& _AccX, double& _AccY, double& _AccZ) {
    // DataChannelService wird asynchron im Hintergrund ausgeführt
    DataChannelService service(SelectedDevice);
    // Warten, bis der Handshake zwischen UDP-host und Device abgeschlossen ist
    usleep(250000);
    if (!service.imuAvailable()) {
        throw SigException("IMU device is not available");
    }

    usleep(20000);

    TimestampedQuaternion quat = service.imuGetRotationQuaternion();
    TimestampedVector lin = service.imuGetLinearAcceleration();

    // Quaternion->Euler
    quat.getRollPitchYaw(_Roll, _Pitch, _Yaw);

    _Roll *= 180.0 / M_PI;
    _Pitch *= 180.0 / M_PI;
    _Yaw *= 180.0 / M_PI;

    _AccX = lin.x();
    _AccY = lin.y();
    _AccZ = lin.z();
}

int SigVisionClass::StartImageTransfer(
    void* _Context,
    GetImageCallback _ImageCallback) {
    if (IsTransferring) {
        throw SigException("Image transfer is already active");
    }

    visiontransfer::AsyncTransfer asyncTransfer(SelectedDevice);

    visiontransfer::ImageSet imageSet;

    ImuData LImuDaten;
    int imgNo = 0;
    IsTransferring = true;
    while (IsTransferring) {
        if (asyncTransfer.collectReceivedImageSet(imageSet, 0.1)) {
            getImuDaten(LImuDaten);
            ImgMetadata Metadata;
            Metadata.Width = imageSet.getWidth();
            Metadata.Height = imageSet.getHeight();
            Metadata.imgSequence = imageSet.getSequenceNumber();
            imageSet.getTimestamp(Metadata.timeSeconds, Metadata.timeMicrosecs);
            Metadata.ExposureTimeuSec = imageSet.getExposureTime();
            imgNo++;
            int cnt = imageSet.getNumberOfImages();
            for (int i = 0; i < cnt; i++) {
                visiontransfer::ImageSet::ImageType imgType = imageSet.getImageType(i);
                visiontransfer::ImageSet::ImageFormat imgFormat = imageSet.getPixelFormat(i);

                Metadata.DataFormat = DATA_FORMAT_MONO8;
                switch (imgFormat) {
                    case visiontransfer::ImageSet::ImageFormat::FORMAT_8_BIT_MONO: {
                        Metadata.DataFormat = DATA_FORMAT_MONO8;
                        break;
                    }
                    case visiontransfer::ImageSet::ImageFormat::FORMAT_8_BIT_RGB: {
                        Metadata.DataFormat = DATA_FORMAT_RGB8;
                        break;
                    }
                    case visiontransfer::ImageSet::ImageFormat::FORMAT_12_BIT_MONO: {
                        Metadata.DataFormat = DATA_FORMAT_MONO12;
                        break;
                    }
                }
                Metadata.RowStride = imageSet.getRowStride(i);
                switch (imgType) {
                    case visiontransfer::ImageSet::ImageType::IMAGE_LEFT: {
                        UINT8* PixelData = imageSet.getPixelData(i);
                        _ImageCallback(_Context, imgNo, LImuDaten, 'L', Metadata, PixelData);
                        break;
                    }
                    case visiontransfer::ImageSet::ImageType::IMAGE_RIGHT: {
                        UINT8* PixelData = imageSet.getPixelData(i);
                        _ImageCallback(_Context, imgNo, LImuDaten, 'R', Metadata, PixelData);
                        break;
                    }
                    case visiontransfer::ImageSet::ImageType::IMAGE_DISPARITY: {
                        UINT8* PixelData = imageSet.getPixelData(i);
                        // imageSet.getQMatrix // disparity to Depth mapping matrix
                        // imageSet.getSubpixelFactor
                        _ImageCallback(_Context, imgNo, LImuDaten, 'D', Metadata, PixelData);

                        Reconstruct3D recon3d;
                        Metadata.DataFormat = DATA_FORMAT_SINGLE;
                        Metadata.RowStride = Metadata.Width * sizeof(float);
                        float* zMap = recon3d.createZMap(imageSet, 0);
                        _ImageCallback(_Context, imgNo, LImuDaten, 'Z', Metadata, zMap);
                        break;
                    }
                }

            }
        }
    } // while IsTransferring
    // Die Schleife wird erst verlassen, wenn ein anderer Thread StopImageTransfer()
    // aufruft, wo dann IsTransferring auf False gesetzt wird.
    return imgNo;
}

void SigVisionClass::StopImageTransfer() {
    if (!IsTransferring) {
        throw SigException("Image transfer is not active");
    }
    IsTransferring = false;
}
