unit u_SigVisionTransferImport;

interface

uses
  Types,
  SysUtils,
  Classes,
  StrUtils,
  u_dzDllLoader,
  u_dzDllApi,
  u_dzErrorThread,
  u_dzLogging,
  u_dzTranslator,
  u_SigLogging,
  u_SigException;

type
  ESigVisionTransfer = class(ESigError)
  end;

type
  TSigVisionTransfer = class(TDzDllApi)
  public
    const
      DLL_Name = 'SigVisionTransfer.dll';
    type
      TSigVisionHandle = packed record
      private
        FValue: Integer;
      public
        function IsValid: Boolean;
        procedure Invalidate;
        property Value: Integer read FValue;
      end;
    type
      TDataFormatEnum = (dfeMono8, dfeRgb8, dfeMono12, dfeSingle);
      TImuData = packed record
        Roll: Double;
        Pitch: Double;
        Yaw: Double;
        AccX: Double;
        AccY: Double;
        AccZ: Double;
      end;
      PImuData = ^TImuData;
      ///<summary>
      /// Metadaten des ImageSets, identisch bei allen Images eines Sets,
      /// mit Ausnahme von DataFormat und RowStride </summary>
      TMetadata = packed record
        Width: Int32;
        Height: Int32;
        imgSequence: UInt32; // Sequence Number (was immer genau das ist))
        timeSeconds: Int32; // Timestamp Seconds-Part
        timeMicrosecs: Int32; // Timestamp Microseconds-Part
        _DataFormat: Int32; // Datenformat des Images siehe DataFormat-Methode
        RowStride: Int32; // Row-Stride des Images (in Bytes)
        ExposureTimeuSec: Int32; // Belichtungszeit
        function DataFormat: TDataFormatEnum;
      end;
      PMetadata = ^TMetadata;
    type
      TGetImageCallbackProc = procedure(_Context: Pointer; _ImgNo: Int32; _ImuDaten: PImuData; _Which: AnsiChar; _Metadata: PMetadata; _Data: PByte); stdcall;
  private
    type
      TSigVision_GetLastError = function(_LastError: PAnsiChar; _MaxLen: Integer): Integer; stdcall;
      TInitialize = function(): Integer; stdcall;
      TFinalize = function(): Integer; stdcall;
      TCreateSigVisionClass = function(var _Handle: TSigVisionHandle): Integer; stdcall;
      TDestroySigVisionClass = function(_Handle: TSigVisionHandle): Integer; stdcall;
      TSigVision_GetIMU = function(_Handle: TSigVisionHandle;
        var _Roll, _Pitch, _Yaw: Double;
        var _AccX, _AccY, _AccZ: Double): Integer; stdcall;
      TSigVision_StartImageTransfer = function(_Handle: TSigVisionHandle;
        _Context: Pointer; _OnGetImage: TGetImageCallbackProc): Integer; stdcall;
      TSigVision_StopImageTransfer = function(_Handle: TSigVisionHandle): Integer; stdcall;
  private
    doGetLastError: TSigVision_GetLastError;
    doInitialize: TInitialize;
    doFinalize: TFinalize;
    doCreateClass: TCreateSigVisionClass;
    doDestroyClass: TDestroySigVisionClass;
    doGetIMU: TSigVision_GetIMU;
    doStartImageTransfer: TSigVision_StartImageTransfer;
    doStopImageTransfer: TSigVision_StopImageTransfer;
  protected
    ///<summary>
    /// Called by the constructor after the dll has been loaded,
    /// to initialize any entry points </summary>
    procedure InitEntryPoints; override;
  public
    constructor Create; overload;
    function CheckSigVisionResult(_Res: Integer; const _Api: string): Integer;
    function GetLastError: string;
    procedure Initialize;
    procedure Finalize;
    function CreateClass: TSigVisionHandle;
    procedure DestroyClass(var _Handle: TSigVisionHandle);
    procedure GetIMU(_Handle: TSigVisionHandle;
      out _Roll, _Pitch, _Yaw: Double;
      out _AccX, _AccY, _AccZ: Double);
    function StartImageTransfer(_Handle: TSigVisionHandle;
      _Context: Pointer; _OnGetImage: TGetImageCallbackProc): Integer;
    function StopImageTransfer(_Handle: TSigVisionHandle): Integer;
  end;

implementation

uses
  u_dzVersionInfo;

{ TSigVisionTransfer }

constructor TSigVisionTransfer.Create;
begin
  MethodTrack('TSigVisionTransfer.Create');

  Create(DLL_Name, False);
  LogDebug('Loaded: ' + FDllLoader.DllFilename
    + ' Version: ' + FDllLoader.DllVersion.FileVersionStr(vpFull));
end;

procedure TSigVisionTransfer.InitEntryPoints;
begin
  inherited InitEntryPoints;
  doGetLastError := FDllLoader.GetProcAddressEx('SigVision_GetLastError');
  doInitialize := FDllLoader.GetProcAddressEx('Initialize');
  doFinalize := FDllLoader.GetProcAddressEx('Finalize');
  doCreateClass := FDllLoader.GetProcAddressEx('CreateSigVisionClass');
  doDestroyClass := FDllLoader.GetProcAddressEx('DestroySigVisionClass');
  doGetIMU := FDllLoader.GetProcAddressEx('SigVision_GetIMU');
  doStartImageTransfer := FDllLoader.GetProcAddressEx('SigVision_StartImageTransfer');
  doStopImageTransfer := FDllLoader.GetProcAddressEx('SigVision_StopImageTransfer');
end;

function TSigVisionTransfer.CheckSigVisionResult(_Res: Integer; const _Api: string): Integer;
var
  Err: string;
begin
  if _Res < 0 then begin
    Err := GetLastError;
    raise ESigVisionTransfer.CreateFmt(_('Error "%s" calling %s'), [Err, _Api]);
  end;
  Result := _Res;
end;

procedure TSigVisionTransfer.Initialize;
begin
  CheckSigVisionResult(doInitialize, 'Initialize');
end;

procedure TSigVisionTransfer.Finalize;
begin
  CheckSigVisionResult(doFinalize, 'Finalize');
end;

function TSigVisionTransfer.CreateClass: TSigVisionHandle;
begin
  CheckSigVisionResult(
    doCreateClass(Result), 'CreateSigVisionClass');
end;

procedure TSigVisionTransfer.DestroyClass(var _Handle: TSigVisionHandle);
begin
  CheckSigVisionResult(doDestroyClass(_Handle), 'DestroySigVisionClass');
  _Handle.Invalidate;
end;

procedure TSigVisionTransfer.GetIMU(_Handle: TSigVisionHandle;
  out _Roll, _Pitch, _Yaw: Double;
  out _AccX, _AccY, _AccZ: Double);
begin
  CheckSigVisionResult(
    doGetIMU(_Handle, _Roll, _Pitch, _Yaw, _AccX, _AccY, _AccZ), 'SigVision_GetIMU');
end;

function TSigVisionTransfer.GetLastError: string;
var
  ErrBuffer: AnsiString;
  Len: Integer;
begin
  SetLength(ErrBuffer, 500);
  Len := doGetLastError(@ErrBuffer[1], Length(ErrBuffer));
  if Len < 0 then
    raise ESigVisionTransfer.CreateFmt(_('Error %d in call to GetLastError'), [Len]);
  SetLength(ErrBuffer, Len);
  Result := string(ErrBuffer);
end;

function TSigVisionTransfer.StartImageTransfer(_Handle: TSigVisionHandle; _Context: Pointer;
  _OnGetImage: TGetImageCallbackProc): Integer;
begin
  Result := CheckSigVisionResult(
    doStartImageTransfer(_Handle, _Context, _OnGetImage), 'SigVision_StartImageTransfer');
end;

function TSigVisionTransfer.StopImageTransfer(_Handle: TSigVisionHandle): Integer;
begin
  Result := CheckSigVisionResult(
    doStopImageTransfer(_Handle), 'SigVision_StopImageTransfer');
end;

{ TSigVisionTransfer.TMetadata }

function TSigVisionTransfer.TMetadata.DataFormat: TDataFormatEnum;
begin
  Result := TDataFormatEnum(_DataFormat);
end;

{ TSigVisionTransfer.TSigVisionHandle }

procedure TSigVisionTransfer.TSigVisionHandle.Invalidate;
begin
  FValue := 0;
end;

function TSigVisionTransfer.TSigVisionHandle.IsValid: Boolean;
begin
  Result := (FValue <> 0);
end;

end.
