unit u_CameraScarlet;

interface

// [...]

type
  TCameraScarlet = class
  public
    type
      TImuData = TSigVisionTransfer.TImuData;
      TMetadata = TSigVisionTransfer.TMetadata;
      TImageEnum = (ieLeft, ieRight, ieDisparity, ieZMap);
      TImageEnumSet = set of TImageEnum;
      TSigVisionHandle = TSigVisionTransfer.TSigVisionHandle;
  private
    type
      TOnGetImageCallbackMethod = procedure(_ImgNo: Integer; const _ImuDaten: TImuData;
        _Which: TImageEnum; const _Metadata: TMetadata; _Data: PByte) of object;
    type
      TImageTransferThread = class(TErrorThread)
      private
        FOwner: TSigVisionTransfer;
        FHandle: TSigVisionHandle;
        FImageTransferActive: Boolean;
        FOnGetImage: TOnGetImageCallbackMethod;
        procedure doGetImageCallback(_ImgNo: Integer; const _ImuDaten: TImuData;
          _Which: AnsiChar; const _Metadata: TMetadata; _Data: PByte);
      protected
        procedure doExecute; override;
        procedure TerminatedSet; override;
      public
        constructor Create(
          _Owner: TSigVisionTransfer;
          _Handle: TSigVisionHandle;
          _OnGetImage: TOnGetImageCallbackMethod);
      end;
  public
  end;

// [...]

implementation

procedure TCameraScarlet.StartImageTransfer(const _Measurement, _BaseFn, _BaseFn2: string;
  _Which: TImageEnumSet);
var
  CaptureFn1: string;
  CaptureFn2: string;
begin
  FLastImgSequence := -1;
  FImagesLost := 0;

  FRecordingTime := TStopwatch.StartNew;

  FMeasurement := _Measurement;
  FBaseFn1 := _BaseFn;
  FBaseFn2 := _BaseFn2;

  CaptureFn2 := '';

  if ieLeft in _Which then begin
    CaptureFn1 := _BaseFn + '.Left.SigBlob';
    if _BaseFn2 <> '' then
      CaptureFn2 := _BaseFn2 + '.Left.SigBlob';
    FImageWriter[ieLeft] := TSigBlobImageWriter.Create(CaptureFn1, CaptureFn2);
  end;

  if ieDisparity in _Which then begin
    CaptureFn1 := _BaseFn + '.Disparity.SigBlob';
    if _BaseFn2 <> '' then
      CaptureFn2 := _BaseFn2 + '.Disparity.SigBlob';
    FImageWriter[ieDisparity] := TSigBlobImageWriter.Create(CaptureFn1, CaptureFn2);
  end;

  if ieZMap in _Which then begin
    CaptureFn1 := _BaseFn + '.ZMap.SigBlob';
    if _BaseFn2 <> '' then
      CaptureFn2 := _BaseFn2 + '.ZMap.SigBlob';
    FImageWriter[ieZMap] := TSigBlobImageWriter.Create(CaptureFn1, CaptureFn2);
  end;

  FTransferThread := TImageTransferThread.Create(FSigVisionTransfer, FHandle, HandleOnGetImage);
end;

procedure TCameraScarlet.StopImageTransfer;
var
  Cam: TImageEnum;
begin
  if Assigned(FTransferThread) then begin
    FTransferThread.Terminate;
    FreeAndNil(FTransferThread);

    for Cam := Low(TImageEnum) to High(TImageEnum) do begin
      if Assigned(FImageWriter[Cam]) then begin
        while not FImageWriter[Cam].TryClose do
          LogInfo(_('Closing file'));
        FreeAndNil(FImageWriter[Cam]);
        FRecordingTime.Stop;
      end;
    end;
  end;
end;

procedure OnGetImageCallback(_Context: Pointer;
  _ImgNo: Integer; _ImuDaten: TSigVisionTransfer.PImuData; _Which: AnsiChar;
  _Metadata: TSigVisionTransfer.PMetadata; _Data: PByte); stdcall;
var
  Thread: TCameraScarlet.TImageTransferThread absolute _Context;
begin
  Thread.doGetImageCallback(_ImgNo, _ImuDaten^, _Which, _Metadata^, _Data);
end;

{ TCameraScarlet.TImageTransferThread }

constructor TCameraScarlet.TImageTransferThread.Create(
  _Owner: TSigVisionTransfer;
  _Handle: TSigVisionHandle;
  _OnGetImage: TCameraScarlet.TOnGetImageCallbackMethod);
begin
  FOwner := _Owner;
  FHandle := _Handle;
  FOnGetImage := _OnGetImage;
  inherited Create(False);
end;

procedure TCameraScarlet.TImageTransferThread.doExecute;
begin
  FImageTransferActive := True;
  try
    FOwner.CheckSigVisionResult(
      FOwner.StartImageTransfer(FHandle, Self, OnGetImageCallback),
      'SigVision_StartImageTransfer');
    // Obiger Aufruf kehrt erst zurueck, wenn ein anderer Thread SigVision_StopImageTransfer aufruft.
    // Das macht im einfachsten Fall TCameraScarlet.TImageTransferThread.Destroy
  finally
    FImageTransferActive := False;
  end;
end;

procedure TCameraScarlet.TImageTransferThread.TerminatedSet;
begin
  inherited;
  if Assigned(FOwner) then begin
    if FImageTransferActive then begin
      FImageTransferActive := False;
      FOwner.StopImageTransfer(FHandle);
    end;
  end;
end;

procedure TCameraScarlet.TImageTransferThread.doGetImageCallback(
  _ImgNo: Integer; const _ImuDaten: TImuData; _Which: AnsiChar; const _Metadata: TMetadata; _Data: PByte);
var
  Which: TImageEnum;
begin
  if not Assigned(FOnGetImage) then
    Exit; //==>

  if _Which = 'L' then
    Which := ieLeft
  else if _Which = 'R' then
    Which := ieRight
  else if _Which = 'D' then
    Which := ieDisparity
  else if _Which = 'Z' then
    Which := ieZMap
  else begin
      // ignore
    Exit; //==>
  end;
  FOnGetImage(_ImgNo, _ImuDaten, Which, _Metadata, _Data);
end;

end.